@@ -1,7 +1,8 @@
 library(RGCCTranslationUnit)
 tu = parseTU("tu.c.001t.tu")
 
-file_p = function(src, files, allowBuiltIns = FALSE) grepl("^cu", src)
+file_p = function(src, files, allowBuiltIns = FALSE)
+             grepl("^cu", src)
 r =  getRoutines(tu, checkSourceFile = file_p)
 rr = lapply(r, resolveType, tu)
 
