@@ -22,9 +22,11 @@ cx = copyToDevice(x)
 i = cx[]
 }))
 
- # Yes we can use dnorm( log = TRUE)
+ # Yes we can use dnorm( log = TRUE)!
 r.tm = system.time(replicate(10, log(dnorm(x, mu, sigma))))
 
+cu.tm/r.tm
+
 head(i)
 
 # The values will not be the same as if we had computed them in R.
