@@ -192,6 +192,28 @@ convertDoubleArrayToR_full(int len, const double *x, int copy, int start, int en
 
 #define MIN(a, b)  ((a) < (b) ? (a) : (b))
 
+const char **
+getCharArrayPtr(SEXP r_value)
+{
+    int i;
+    int len;
+    const char **dest;
+    
+    len = Rf_length(r_value);
+
+    if(len == 0)
+	return(NULL);
+
+    dest = (const char **) R_alloc(sizeof(char *), len);
+    if(!dest) {
+	PROBLEM "cannot allocate space for char **"
+        ERROR;
+    }
+    for(i = 0; i < len; i++)
+	dest[i] = CHAR(STRING_ELT(r_value, i));    
+    return(dest);
+}
+
 void
 convertRCharacterToCharArray(char *dest, SEXP r_value, int array_len)
 {
@@ -200,7 +222,7 @@ convertRCharacterToCharArray(char *dest, SEXP r_value, int array_len)
 
     len = MIN(Rf_length(r_value), array_len);
     for(i = 0; i < len; i++)
-	dest[i] = CHAR(STRING_ELT(r_value, i))[0];
+	dest[i] = CHAR(STRING_ELT(r_value, i))[0];//XXX should this [0] be here?
 }
 
 SEXP
@@ -569,7 +591,7 @@ R_addressOfPointer(SEXP ext)
 /* Need to know the tag and the protected for the external pointer. 
 Leave to R code to put in the correct class.
 */
-   void *p = R_ExternalPtrAddr(ext);
+//   void *p = R_ExternalPtrAddr(ext);
 	
    return(R_NilValue);
 }
@@ -667,7 +689,7 @@ R_makeEnumValue(int val, const char *elName, const char *className)
     PROTECT(ans = ScalarInteger(val));
     SET_NAMES(ans, mkString(elName));
     PROTECT(klass = NEW_CHARACTER(2));
-    SET_STRING_ELT(klass, 0, mkChar("CXLanguageKind"));
+    SET_STRING_ELT(klass, 0, mkChar(className));
     SET_STRING_ELT(klass, 1, mkChar("EnumValue"));
     SET_CLASS(ans, klass);
     UNPROTECT(2);
@@ -676,3 +698,31 @@ R_makeEnumValue(int val, const char *elName, const char *className)
     
     return(ans);
 }
+
+
+#ifndef LOCAL_CREATE_REF
+SEXP
+R_createReference(void *ptr, const char * const className, const char * tag)
+{
+    SEXP ans, klass;
+    PROTECT(klass = MAKE_CLASS(className));
+    PROTECT(ans = NEW_OBJECT(klass));
+    if(!tag)
+        tag = className;
+    SET_SLOT(ans, Rf_install("ref"), R_MakeExternalPtr(ptr, Rf_install(tag), R_NilValue));
+    UNPROTECT(2);
+    return(ans);
+}
+#endif
+
+
+void 
+copyRVectorToDoubleArray(SEXP r_vec, double *dest, int numEls)
+{
+    int i;
+    int num = MIN(numEls, Rf_length(r_vec));
+    double *els = REAL(r_vec);
+
+    for(i = 0 ; i < num; i++)
+	dest[i] = els[i];
+}
