@@ -1,5 +1,6 @@
 #include "RCUDA.h"
 
+
 SEXP R_cudaGetLastError();
 
 SEXP
@@ -14,15 +15,28 @@ R_cudaErrorInfo(CUresult val)
 }
 
 
+SEXP
+R_cudaError_t_Info(cudaError_t val)
+{
+    SEXP ans;
+    PROTECT(ans = ScalarInteger(val));
+    SET_CLASS(ans, mkString("cudaError_t"));
+//???
+    SET_NAMES(ans, R_cudaGetLastError());
+    UNPROTECT(1);
+    return(ans);
+}
+
+
 /* 
  We want this to be a high-level routine that can allocate space 
  in the appropriate device and then make the copy.
  Potentially, we should put information about the size and type 
  on the return value so that we can recover the data when we want
  to copy it back.
- */
+*/
 SEXP
-R_cudaMemcpy(SEXP r_src,  SEXP r_ptr)
+R_cudaMemcpy(SEXP r_src,  SEXP r_ptr, SEXP r_elSize)
 {
     SEXP ans = R_NilValue;
     void *ptr = getRReference(r_ptr);
@@ -39,11 +53,16 @@ R_cudaMemcpy(SEXP r_src,  SEXP r_ptr)
 	  break;
     case REALSXP:
     {
-	elSize = 4;
+      elSize = INTEGER(r_elSize)[0];
+      if(elSize == 8)
+	data = REAL(r_src);
+      else {
+        elSize = 4;
 	float *fl = (float *) R_alloc(len, elSize);
 	for(i = 0; i < len; i++)
 	    fl[i] = REAL(r_src)[i];
 	  data = fl;
+      }
     }
           break;
     default:
@@ -52,9 +71,9 @@ R_cudaMemcpy(SEXP r_src,  SEXP r_ptr)
 
     }
 
-    CUresult status = cudaMemcpy(ptr, data, elSize * len, cudaMemcpyHostToDevice);
+    cudaError_t status = cudaMemcpy(ptr, data, elSize * len, cudaMemcpyHostToDevice);
     if(status)
-	return(R_cudaErrorInfo(status));
+	return(R_cudaError_t_Info(status));
 
     return(ans);
 }
@@ -76,7 +95,12 @@ convertRObjToGPU(SEXP arg, float *floatLoc, void **argLoc)
     int ty = TYPEOF(arg);
 
     if(ty == EXTPTRSXP) {
-        argLoc[0] =  R_ExternalPtrAddr(arg);
+        argLoc[0] = R_ExternalPtrAddr(arg);
+	return(argLoc);
+    }
+
+    if(arg == R_NilValue) {
+	argLoc[0] = NULL;
 	return(argLoc);
     }
 
@@ -91,25 +115,38 @@ convertRObjToGPU(SEXP arg, float *floatLoc, void **argLoc)
 	float *gpu_fl;
 	for(i = 0 ; i < len; i++)
 	    fl[i] = REAL(arg)[i];
-	CUresult status = cudaMalloc((void **)  &gpu_fl, len * sizeof(float));
+	cudaError_t status = cudaMalloc((void **)  &gpu_fl, len * sizeof(float));
 	if(status) {
 	    PROBLEM "cannot allocate memory on the GPU %d", status
 		ERROR;
 	}
-	cudaMemcpy(gpu_fl, fl, len * sizeof(float),  cudaMemcpyHostToDevice);
+	status = cudaMemcpy(gpu_fl, fl, len * sizeof(float),  cudaMemcpyHostToDevice);
+	if(status) {
+	    if(gpu_fl) 
+               cudaFree(gpu_fl);
+	    PROBLEM "cannot copy memory on the GPU %d", status
+		ERROR;
+	}
 
     } else if(ty == INTSXP) {
-	if(len == 1)
+        if(len == 1) 
 	    return(INTEGER(arg));
+
 	int *gpu_int = NULL;
 	void *p;
-	CUresult status = cudaMalloc((void **)  &p, len * sizeof(int));
+	cudaError_t status = cudaMalloc((void **)  &p, len * sizeof(int));
 	if(status) {
 	    PROBLEM "cannot allocate memory on the GPU %d", status
 		ERROR;
 	}
 	gpu_int = (int *) p;
-	cudaMemcpy(gpu_int, INTEGER(arg), len * sizeof(int),  cudaMemcpyHostToDevice);	
+	status = cudaMemcpy(gpu_int, INTEGER(arg), len * sizeof(int),  cudaMemcpyHostToDevice);	
+	if(status) {
+	    if(p)
+               cudaFree(p);
+	    PROBLEM "cannot copy memory on the GPU %d", status
+		ERROR;
+	}
 	return(gpu_int);
     }
 
@@ -126,11 +163,11 @@ R_cuLaunchKernel(SEXP r_fun, SEXP r_gridDims, SEXP r_blockDims, SEXP r_args, SEX
     int *gridDims, *blockDims;
     gridDims = INTEGER(r_gridDims);
     blockDims = INTEGER(r_blockDims);
-    CUfunction fun = GET_REF(r_fun, CUfunction);
+    CUfunction fun = (CUfunction) getRReference(r_fun); // was  GET_REF(r_fun, CUfunction) but about to change GET_REF.
     CUstream stream = NULL;
 
     int nargs = Rf_length(r_args), i;
-         //XXX if we do an asynchronous launch, then this memory will disappear when we exit this call and the code is still running.
+      //XXX if we do an asynchronous launch, then this memory will disappear when we exit this call and the code is still running.
 #if 1
     void **args, **args2; //set from r_args
     args = (void **) R_alloc(nargs, sizeof(void*));  
@@ -138,9 +175,11 @@ R_cuLaunchKernel(SEXP r_fun, SEXP r_gridDims, SEXP r_blockDims, SEXP r_args, SEX
     float *floats = (float *) R_alloc(nargs, sizeof(float)); 
     for(i = 0; i < nargs; i++) {
 	SEXP arg = VECTOR_ELT(r_args, i);
-	/* If we have a scalar, we pass the address of that scalar. For a REAL, we have to put it into a float and use the address of that float */
+	args[i] = args2[i] = NULL;
+	/* If we have a scalar, we pass the address of that scalar. 
+           For a REAL, we have to put it into a float and use the address of that float */
 	args[i] = convertRObjToGPU(arg, floats + i, args2 + i);
-//	fprintf(stderr, "arg %d = %p, %p\n", i, args[i], args2[i]);
+	//	fprintf(stderr, "arg %d = %p     (%p)\n", i, args[i], args2[i]);
     }
 #else
     void *args[4];
@@ -156,6 +195,9 @@ R_cuLaunchKernel(SEXP r_fun, SEXP r_gridDims, SEXP r_blockDims, SEXP r_args, SEX
 
 #endif
 
+    if(Rf_length(r_stream))
+      stream = (CUstream) getRReference(r_stream);
+
     CUresult status = cuLaunchKernel(fun, gridDims[0], gridDims[1], gridDims[2], blockDims[0], blockDims[1], blockDims[2], INTEGER(r_sharedMemBytes)[0], stream, args, NULL);
     if(status != CUDA_SUCCESS) {
 	PROBLEM "error launching CUDA kernel %d", status
@@ -182,7 +224,7 @@ R_loadModule(SEXP r_filename)
 SEXP
 R_Module_getFunction(SEXP r_module, SEXP r_name)
 {
-    CUmodule mod = GET_REF(r_module, CUmodule);
+    CUmodule mod = (CUmodule) getRReference(r_module); //XX was GET_REF(r_module, CUmodule);
     CUfunction func;
     CUresult status = cuModuleGetFunction(&func, mod, CHAR(STRING_ELT(r_name, 0)));
     if(status != CUDA_SUCCESS) {
@@ -200,7 +242,7 @@ SEXP
 R_createContext(SEXP r_flags, SEXP r_dev)
 {
    CUcontext ctxt;
-   CUresult status =  cuCtxCreate(&ctxt, INTEGER(r_flags)[0], INTEGER(r_dev)[0]);
+   CUresult status = cuCtxCreate(&ctxt, INTEGER(r_flags)[0], INTEGER(r_dev)[0]);
     if(status != CUDA_SUCCESS) {
 	return(ScalarInteger(status));
 
@@ -238,64 +280,37 @@ SEXP
 R_cudaMalloc(SEXP r_numBytes)
 {
     void *ptr = NULL;
-    CUresult status = cudaMalloc(&ptr, INTEGER(r_numBytes)[0]);
+    cudaError_t status = cudaMalloc(&ptr, REAL(r_numBytes)[0]);
     if(status) {
-	return(ScalarInteger(status));
+	return(R_cudaError_t_Info(status));
     }
     return(R_createReference(ptr, "cudaPtr", "cudaPtr", R_cudaFree));
 }
 
 
-
-SEXP 
-R_cuDeviceGet(SEXP which)
-{
-    CUdevice cu_device;
-    CUresult err = cuDeviceGet(&cu_device, INTEGER(which)[0]);
-    if(err) {
-	PROBLEM "problem getting device"
-	    ERROR;
-    }
-    return(ScalarInteger(cu_device));
-}
-
-
 SEXP
 R_cudaSetDevice(SEXP r_dev)
 {
-    CUresult status = cudaSetDevice(INTEGER(r_dev)[0]);
-    return(ScalarInteger(status));
+    cudaError_t status = cudaSetDevice(INTEGER(r_dev)[0]);
+    return(R_cudaErrorInfo(status));
 }
 
 
 SEXP
-R_cuInit(SEXP r_flags)
+R_cuInit(SEXP r_flags, SEXP r_asCUresult)
 {
     CUresult status = cuInit(INTEGER(r_flags)[0]);
-    return(ScalarInteger(status));
+    return(LOGICAL(r_asCUresult)[0] ? Renum_convert_CUresult(status) : ScalarInteger(status));
 }
 
 
-SEXP
-R_cuCtxGetCurrent()
-{
-  CUcontext ctx = NULL;
-  CUresult status = cuCtxGetCurrent(&ctx);
-  if(status) 
-      return(ScalarInteger(status));
-  
-  if(!ctx)
-      return(R_NilValue);
-
-  return(R_createRef(ctx, "CUcontext"));  
-}
 
 SEXP
 R_cuGetVersion()
 {
   SEXP ans = NEW_INTEGER(2);
   int version;
-  CUresult status = cudaDriverGetVersion(&version);
+  cudaError_t status = cudaDriverGetVersion(&version);
   INTEGER(ans)[0] = version;
   status = cudaRuntimeGetVersion(&version);
   INTEGER(ans)[1] = version;
@@ -316,20 +331,6 @@ R_cuDriverGetVersion()
 
 
 
-SEXP
-R_cuCtxDestroy(SEXP r_ctx)
-{
-    if(TYPEOF(r_ctx) == EXTPTRSXP) {
-	r_ctx = GET_SLOT(r_ctx, Rf_install("ref"));
-    }
-
-    CUcontext ctx = (CUcontext) R_ExternalPtrAddr(r_ctx);
-    if(ctx) {
-	cuCtxDestroy(ctx);
-	R_SetExternalPtrAddr(r_ctx, NULL);
-    }
-    return(R_NilValue);
-}
 
 
 SEXP
@@ -339,9 +340,23 @@ R_getCudaIntVector(SEXP r_ptr, SEXP r_len)
     SEXP ans = NEW_INTEGER(len);
     void *ptr = getRReference(r_ptr);
 
-    CUresult status = cudaMemcpy(INTEGER(ans), ptr, len * sizeof(int), cudaMemcpyDeviceToHost);
+    cudaError_t status = cudaMemcpy(INTEGER(ans), ptr, len * sizeof(int), cudaMemcpyDeviceToHost);
     if(status) 
-	return(R_cudaErrorInfo(status));
+	return(R_cudaError_t_Info(status));
+
+    return(ans);
+}
+
+SEXP
+R_getCudaDoubleVector(SEXP r_ptr, SEXP r_len)
+{
+    int len = INTEGER(r_len)[0];
+    SEXP ans = NEW_NUMERIC(len);
+    void *ptr = getRReference(r_ptr);
+
+    cudaError_t status = cudaMemcpy(REAL(ans), ptr, len * sizeof(double), cudaMemcpyDeviceToHost);
+    if(status) 
+	return(R_cudaError_t_Info(status));
 
     return(ans);
 }
@@ -356,12 +371,13 @@ R_getCudaFloatVector(SEXP r_ptr, SEXP r_len, SEXP r_indices)
 
     float *fl = (float *) R_alloc(len, sizeof(float));
     if(!fl) {
-	PROBLEM "..." ERROR;
+      PROBLEM "cannot allocate space for the array of %d floats", len
+      ERROR;
     }
 
-    CUresult status = cudaMemcpy(fl, ptr, len * sizeof(int), cudaMemcpyDeviceToHost);
+    cudaError_t status = cudaMemcpy(fl, ptr, len * sizeof(int), cudaMemcpyDeviceToHost);
     if(status) 
-	return(R_cudaErrorInfo(status));
+	return(R_cudaError_t_Info(status));
 
     
     if(Rf_length(r_indices)) {
@@ -381,9 +397,6 @@ R_getCudaFloatVector(SEXP r_ptr, SEXP r_len, SEXP r_indices)
 
 
 
-
-
-
 SEXP
 R_cudaGetLastError()
 {
@@ -433,3 +446,121 @@ R_cuMemGetInfo()
     REAL(ans)[1] = total;
     return(ans);
 }
+
+
+
+
+
+#if 0  				/* Now in context.c */
+SEXP
+R_cuCtxDestroy(SEXP r_ctx)
+{
+    if(TYPEOF(r_ctx) == EXTPTRSXP) {
+	r_ctx = GET_SLOT(r_ctx, Rf_install("ref"));
+    }
+
+    CUcontext ctx = (CUcontext) R_ExternalPtrAddr(r_ctx);
+    if(ctx) {
+	cuCtxDestroy(ctx);
+	R_SetExternalPtrAddr(r_ctx, NULL);
+    }
+    return(R_NilValue);
+}
+
+SEXP
+R_cuCtxGetCurrent(SEXP r_asContext)
+{
+  CUcontext ctx = NULL;
+  CUresult status = cuCtxGetCurrent(&ctx);
+  if(status) 
+      return(ScalarInteger(status));
+  
+  if(!ctx)
+      return(R_NilValue);
+
+  return(R_createRef(ctx, "CUcontext"))
+}
+
+
+SEXP 
+R_cuDeviceGet(SEXP which)
+{
+    CUdevice cu_device;
+    CUresult err = cuDeviceGet(&cu_device, INTEGER(which)[0]);
+    if(err) {
+	PROBLEM "problem getting device"
+	    ERROR;
+    }
+    return(ScalarInteger(cu_device));
+}
+
+#endif
+
+/* Test to see if this doesn't work in C code either!, i.e the auto-generated R code gives the same error. 
+Seems to! 
+*/
+SEXP
+R_test_cuCtxGetLimit()
+{
+    CUcontext ctx;
+    CUresult status = cuCtxGetCurrent(&ctx);
+    if(status) {
+	PROBLEM "can't get current %s", cudaGetErrorString(cudaGetLastError())
+	    ERROR;
+    }
+    size_t size;
+    status = cuCtxGetLimit(&size, CU_LIMIT_STACK_SIZE);
+    if(status) {
+	PROBLEM "can't get limit: %s", cudaGetErrorString(cudaGetLastError())
+	    ERROR;
+    }
+    return(ScalarReal(size));
+}
+
+
+
+
+SEXP
+R_cuModuleLoadDataEx(SEXP r_image, SEXP r_Options, SEXP retOpts)
+{
+    SEXP r_ans = R_NilValue;
+    CUmodule module;
+
+
+    const void * image;
+    if(TYPEOF(r_image) == RAWSXP)
+	image = RAW(r_image);
+    else 
+	image = (void *) CHAR(STRING_ELT(r_image, 0));
+    
+    unsigned int numOptions = Rf_length(r_Options);
+    CUresult ans;
+
+    ans = cuModuleLoadDataEx( &module, image, numOptions, INTEGER(r_Options), NULL);
+
+    if(ans)
+       return(R_cudaErrorInfo(ans));
+
+    r_ans = R_createRef(module, "CUmodule");
+
+    return(r_ans);
+}
+
+
+SEXP
+R_cudaThreadSynchronize()
+{
+  cudaError_t ans = cudaThreadSynchronize();
+  return(Renum_convert_cudaError_t(ans));
+}
+
+
+/* To bypass the seg fault in CUDA SDK 5.5 when we unload the DLL w/o having done much. */
+void
+R_initForCUDAFiveFive()
+{
+  CUcontext ctxt;
+  cuCtxCreate(&ctxt, 0L, 0L);
+  void *f;
+  cudaMalloc(&f, 4);
+}
