@@ -1,10 +1,10 @@
 library(RCUDA)
-.C("cuInit")
-
+cuInit()
+if(FALSE) {
 #cudaGetDeviceProperties
 prop = new_cudaDeviceProp()
 ans = cudaGetDeviceProperties(prop, 0L)
 ans
 
 ans = cudaGetDeviceProperties(NULL, 0L)
-
+}
